from charmhelpers import fetch
from charmhelpers.core import host, hookenv, unitdata
from charmhelpers.core.templating import render
from charms.reactive import when, when_not, when_any, when_file_changed,when_not_all, remove_state, set_state, hook

SNMPD_CONF = '/etc/snmp/snmpd.conf'


@when_not('snmpd.installed',
          'snmpd.stopped')
def install_snmpd():
    install_apt_packages()
    set_state('snmpd.installed')


@when('snmpd.installed')
def start_snmpd():
    update_snmpd_conf()
    if not host.service_running('snmpd'):
        hookenv.log('Starting snmpd...')
        host.service_start('snmpd')
    else:
        hookenv.log('Reloading snmpd config...')
        host.service_reload('snmpd')
    hookenv.status_set('active', 'Ready')


@when('config.changed.extra_packages')
def install_apt_packages():
    config = hookenv.config()
    packages = ['snmpd']
    if config['extra_packages']:
        packages += [p.strip() for p in config['extra_packages'].split(',')]
    hookenv.status_set('maintenance', 'Installing packages')
    hookenv.log("Installing apt packages " + str([pkg for pkg in packages]), hookenv.INFO)
    fetch.configure_sources()
    fetch.apt_update()
    fetch.apt_install(packages)
# TODO: remove unnecessary packages


@when_file_changed(SNMPD_CONF, hash_type='sha256')
def update_snmpd_conf():
    config = hookenv.config()
    hookenv.status_set('maintenance', 'Updating snmpd configuration.')

    options = {
        'sysLocation': config['sysLocation'],
        'sysContact': config['sysContact'],
        'acl_config': [acl.strip() for acl in config['acl_config'].splitlines()],
        'other_config': [other.strip() for other in config['other_config'].splitlines()]
    }

    render(
        'snmpd.conf',
        SNMPD_CONF,
        options,
        owner='root',
        group='root'
    )

    hookenv.status_set('maintenance', 'snmpd.conf updated')


@when_any('host.available', 'host.connected')
def host_available():
    pass


@hook('stop')
def remove_snmpd():
    """
    Remove installed packages when the principal charm relation revoked
    """
    if host.service_running('snmpd'):
        hookenv.log('Stopping snmpd...')
        set_state('snmpd.stopped')
    if not host.service_running('snmpd'):
        set_state('snmpd.stopped')
    uninstall_packages()


def uninstall_packages():
    kv = unitdata.kv()
    packages = ['snmpd']
    # if kv.get('extra_packages'):
    #     packages += [p.strip() for p in kv.get('extra_packages').split(',')]
    hookenv.log("Uninstalling.. " + str([pkg for pkg in packages]), hookenv.INFO)
    fetch.apt_purge(packages)
